CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
113 lines
4.1 KiB
C++
113 lines
4.1 KiB
C++
// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
|
|
//
|
|
// Permission is hereby granted, free of charge, to any person obtaining a copy
|
|
// of this software and associated documentation files (the "Software"), to deal
|
|
// in the Software without restriction, including without limitation the rights
|
|
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
|
|
// copies of the Software, and to permit persons to whom the Software is
|
|
// furnished to do so, subject to the following conditions:
|
|
//
|
|
// The above copyright notice and this permission notice shall be included in
|
|
// all copies or substantial portions of the Software.
|
|
//
|
|
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
|
|
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
|
|
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
|
|
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
|
|
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
|
|
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
|
|
// THE SOFTWARE.
|
|
|
|
#include <iostream>
|
|
#include <vector>
|
|
#include <numeric>
|
|
#include <algorithm>
|
|
|
|
#include <hip/hip_runtime.h>
|
|
#include <rocprim/rocprim.hpp>
|
|
|
|
#define HIP_CHECK(condition) \
|
|
do \
|
|
{ \
|
|
hipError_t error = condition; \
|
|
if(error != hipSuccess) \
|
|
{ \
|
|
std::cout << "Error " << hipGetErrorName(error) << '(' << error << ')' << ": " \
|
|
<< hipGetErrorString(error) << " in " << __func__ << " at " << __FILE__ \
|
|
<< ':' << __LINE__ << '\n'; \
|
|
exit(error); \
|
|
} \
|
|
} \
|
|
while(false)
|
|
|
|
int main(int, char**)
|
|
{
|
|
using T = unsigned int;
|
|
|
|
// host input/output
|
|
const size_t size = 1024 * 256;
|
|
std::vector<T> input(size, 1);
|
|
T output = 0;
|
|
|
|
// device input/output
|
|
T * d_input;
|
|
T * d_output;
|
|
HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&d_input), input.size() * sizeof(T)));
|
|
HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&d_output), sizeof(T)));
|
|
HIP_CHECK(
|
|
hipMemcpy(
|
|
d_input, input.data(),
|
|
input.size() * sizeof(T),
|
|
hipMemcpyHostToDevice
|
|
)
|
|
);
|
|
HIP_CHECK(hipDeviceSynchronize());
|
|
|
|
// Calculate expected results on host
|
|
auto expected = std::accumulate(input.begin(), input.end(), 0U);
|
|
|
|
// Temporary storage
|
|
size_t temp_storage_size_bytes;
|
|
void * d_temp_storage = nullptr;
|
|
// Get size of d_temp_storage
|
|
HIP_CHECK(
|
|
rocprim::reduce(
|
|
d_temp_storage, temp_storage_size_bytes,
|
|
d_input, d_output, input.size()
|
|
)
|
|
);
|
|
|
|
// Allocate temporary storage
|
|
HIP_CHECK(hipMalloc(&d_temp_storage, temp_storage_size_bytes));
|
|
HIP_CHECK(hipDeviceSynchronize());
|
|
|
|
// Run
|
|
HIP_CHECK(
|
|
rocprim::reduce(
|
|
d_temp_storage, temp_storage_size_bytes,
|
|
d_input, d_output, input.size()
|
|
)
|
|
);
|
|
HIP_CHECK(hipDeviceSynchronize());
|
|
|
|
// Copy output to host
|
|
HIP_CHECK(
|
|
hipMemcpy(
|
|
&output, d_output,
|
|
sizeof(T),
|
|
hipMemcpyDeviceToHost
|
|
)
|
|
);
|
|
HIP_CHECK(hipDeviceSynchronize());
|
|
|
|
if(output != expected)
|
|
{
|
|
std::cout
|
|
<< "Failure: output (" << output
|
|
<< ") != expected (" << expected << ")"
|
|
<< std::endl;
|
|
return 1;
|
|
}
|
|
return 0;
|
|
}
|